#include "func.h"

Const::Const(double v): v(v)
{}

double Const::calc(double)
{
    return v;
}

double Var::calc(double x)
{
    return x;
}

Unary::Unary(Func *f): f(f)
{}

Unary::Unary(int k): f(new Const(k))
{}

Negate::Negate(Func *f): Unary(f)
{}

Negate::Negate(int k): Unary(k)
{}

double Negate::calc(double x)
{
    return -f -> calc(x);
}

Abst::Abst(Func *f): Unary(f)
{}

Abst::Abst(int k): Unary(k)
{}

double Abst::calc(double x)
{
    double r = f -> calc(x);
    return (r < 0) ? -r : r;
}

Sqrt::Sqrt(Func *f): Unary(f)
{}

Sqrt::Sqrt(int k): Unary(k)
{}

double Sqrt::calc(double x)
{
    return sqrt(f -> calc(x));
}

Sin::Sin(Func *f): Unary(f)
{}

Sin::Sin(int k): Unary(k)
{}

double Sin::calc(double x)
{
    return sin(f -> calc(x));
}

Cos::Cos(Func *f): Unary(f)
{}

Cos::Cos(int k): Unary(k)
{}

double Cos::calc(double x)
{
    return cos(f -> calc(x));
}

Tg::Tg(Func *f): Unary(f)
{}

Tg::Tg(int k): Unary(k)
{}

double Tg::calc(double x)
{
    double v = f -> calc(x);
    return sin(v) / cos(v);
}

Ctg::Ctg(Func *f): Unary(f)
{}

Ctg::Ctg(int k): Unary(k)
{}

double Ctg::calc(double x)
{
    double v = f -> calc(x);
    return cos(v) / sin(v);
}

Ln::Ln(Func *f): Unary(f)
{}

Ln::Ln(int k): Unary(k)
{}

double Ln::calc(double x)
{
    return log(f -> calc(x));
}

Exp::Exp(Func *f): Unary(f)
{}

Exp::Exp(int k): Unary(k)
{}

double Exp::calc(double x)
{
    return exp(f -> calc(x));
}

Binary::Binary(Func *left, Func *right): left(left), right(right)
{}

Binary::Binary(int k, Func *f): Binary(new Const(k), f)
{}

Binary::Binary(Func *f, int k): Binary(f, new Const(k))
{}

Add::Add(Func *left, Func *right): Binary(left, right)
{}

Add::Add(int k, Func *f): Binary(k, f)
{}

double Add::calc(double x)
{
    return left -> calc(x) + right -> calc(x);
}

Sub::Sub(Func *left, Func *right): Binary(left, right)
{}

Sub::Sub(int k, Func *f): Binary(k, f)
{}

double Sub::calc(double x)
{
    return left -> calc(x) - right -> calc(x);
}

Mult::Mult(Func *left, Func *right): Binary(left, right)
{}

Mult::Mult(int k, Func *f): Binary(k, f)
{}

double Mult::calc(double x)
{
    return left -> calc(x) * right -> calc(x);
}

Divide::Divide(Func *left, Func *right): Binary(left, right)
{}

Divide::Divide(int k, Func *f): Binary(k, f)
{}

double Divide::calc(double x)
{
    return left -> calc(x) / right -> calc(x);
}

Power::Power(Func *left, Func *right): Binary(left, right)
{}

Power::Power(int k, Func *f): Binary(k, f)
{}

Power::Power(Func *f, int k): Binary(f, k)
{}

double Power::calc(double x)
{
    return pow(left -> calc(x), right -> calc(x));
}
