#pragma once

#include "math.h"

struct Func
{
    virtual double calc(double x) = 0;
};

struct Const: Func
{
    double v;
    Const(double v);
    double calc(double);
};

struct Var: Func
{
    double calc(double x);
};

struct Unary: Func
{
    Func *f;
    Unary(Func *f);
    Unary(int k);
};

struct Negate: Unary
{
    Negate(Func *f);
    Negate(int k);
    double calc(double x);
};

struct Abst: Unary
{
    Abst(Func *f);
    Abst(int k);
    double calc(double x);
};

struct Sqrt: Unary
{
    Sqrt(Func *f);
    Sqrt(int k);
    double calc(double x);
};

struct Sin: Unary
{
    Sin(Func *f);
    Sin(int k);
    double calc(double x);
};

struct Cos: Unary
{
    Cos(Func *f);
    Cos(int k);
    double calc(double x);
};

struct Tg: Unary
{
    Tg(Func *f);
    Tg(int k);
    double calc(double x);
};

struct Ctg: Unary
{
    Ctg(Func *f);
    Ctg(int k);
    double calc(double x);
};

struct Ln: Unary
{
    Ln(Func *f);
    Ln(int k);
    double calc(double x);
};

struct Exp: Unary
{
    Exp(Func *f);
    Exp(int k);
    double calc(double x);
};

struct Binary: Func
{
    Func *left, *right;
    Binary(Func *left, Func *right);
    Binary(int k, Func *f);
    Binary(Func *f, int k);
};

struct Add: Binary
{
    Add(Func *left, Func *right);
    Add(int k, Func *f);
    double calc(double x);
};

struct Sub: Binary
{
    Sub(Func *left, Func *right);
    Sub(int k, Func *f);
    double calc(double x);
};

struct Mult: Binary
{
    Mult(Func *left, Func *right);
    Mult(int k, Func *f);
    double calc(double x);
};

struct Divide: Binary
{
    Divide(Func *left, Func *right);
    Divide(int k, Func *f);
    double calc(double x);
};

struct Power: Binary
{
    Power(Func *left, Func *right);
    Power(int k, Func *f);
    Power(Func *f, int k);
    double calc(double x);
};
