#include <stdio.h>
#include "func.h"
#include "parser.h"

template<typename T>
inline void _display(T *start, unsigned len, const char *prf = "%u", const char *del = ", ")
{
    if(len==0)
        return;
    printf(prf, *start++);
    while(len-->1)
    {
        fputs(del, stdout);
        printf(prf, *start++);
    }
    putchar('\n');
}

double *fdm(Func *fa, Func *fb, Func *fc, Func *fd, double lb, double rb, double lvalue, double rvalue, unsigned points)
{
    double step = (rb - lb) / ++points;
    Func *st = new Const(step);
    Func *prev = new Sub(new Mult(2, fa), new Mult(fb, st));
    Func *curr = new Add(new Mult(-4, fa), new Mult(new Mult(2, fc), new Power(st, 2)));
    Func *next = new Add(new Mult(2, fa), new Mult(fb, st));
    Func *e = new Mult(new Mult(2, fd), new Power(st, 2));
    double *a = new double[points + 1], *b = new double[points];
    a[0] = 0;
    b[0] = lvalue;
    for(unsigned i = 1;i < points;i++)
    {
        double x = lb + i * step;
        a[i] = -next -> calc(x) / (prev -> calc(x) * a[i - 1] + curr -> calc(x));
        b[i] = (e -> calc(x) - prev -> calc(x) * b[i - 1]) / (prev -> calc(x) * a[i - 1] + curr -> calc(x));
    }
    a[points] = rvalue;
    while(points-->0)
        a[points] = a[points] * a[points + 1] + b[points];
    delete [] b;
    return a;
}

int main()
{
    unsigned pointsAmount = 6;
    Func *a = new Const(1);
    Func *b = parse("-2/(1+2*x)");
    Func *c = parse("-12/(1+2*x)^2");
    Func *d = parse("(1+3*x)/(1+2*x)**2");
    double *h = fdm(a, b, c, d, 2, 6, 4, 8, pointsAmount - 2);
    _display(h, pointsAmount, "%0.3f");
    return 0;
}
